#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QSerialPortInfo>
#include <QtEndian>

#include <QDebug>

#define JPKT_ON              0x01    /* turn on/off dds                                 */
#define JPKT_FREQ            0x02    /* set freq of dds                                 */
#define JPKT_PHASE           0x03    /* set phase of dds                                */
#define JPKT_Data            0x05    /* Data to arduino                                 */
#define JPKT_REQ             0x06    /* packet from arduino saying it wants more data   */
#define JPKT_DATA_ACK        0x07    /* ACK recept for given data                       */
#define JPKT_DATA_NACK       0x08    /* NACK recept for given data (no room)            */
#define JPKT_SYMBOL_PERIOD   0x09    /* set symbol period in microseconds               */


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect(&slip,SIGNAL(rxpacket(QByteArray)),this,SLOT(handlerxpacket(QByteArray)));
    fillPortsInfo();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::handlerxpacket(QByteArray pkt)
{
    uchar pkt_type=pkt[0];
    pkt.remove(0,1);
    switch(pkt_type)//switch on packet type
    {
    case JPKT_FREQ:
        if(pkt.length()==(4))
        {
            qint32 freq=qFromLittleEndian<qint32>((uchar*)pkt.data());
            ui->statusBar->showMessage(((QString)"Freq on DDS is %1").arg(freq));
            ui->listWidget->addItem(((QString)"Freq on DDS is %1").arg(freq));
        }
        break;
    case JPKT_PHASE:
        if(pkt.length()==(4))
        {
            qint32 phase=qFromLittleEndian<qint32>((uchar*)pkt.data());
            ui->statusBar->showMessage(((QString)"Phase on DDS is %1").arg(phase));
            ui->listWidget->addItem(((QString)"Phase on DDS is %1").arg(phase));
        }
        break;
    case JPKT_ON:
        if(pkt.length()==(1))
        {
            if(pkt[0])
            {
                ui->statusBar->showMessage("DDS is on");
                ui->listWidget->addItem("DDS is on");
            }
            else
            {
                ui->statusBar->showMessage("DDS is off");
                ui->listWidget->addItem("DDS is off");
                if(symbolbuffer.length())
                {
                    ui->listWidget->addItem("Symbol buffer not empty");
                }
            }
        }
        break;
    case JPKT_REQ:
        if(pkt.length()==(1))
        {
            uchar byteswanted=pkt[0];
            //ui->statusBar->showMessage(((QString)"Arduino wants %1 more bytes").arg(byteswanted));
            //ui->listWidget->addItem(((QString)"Arduino wants %1 more bytes").arg(byteswanted));

            //send some packets to the arduino if we have it
            if((byteswanted>1)&&(symbolbuffer.length()))//we need at least 2 bytes to send data
            {
                //que up some symbols
                QByteArray ba;
                ba.push_back((char)JPKT_Data);
                ba.push_back(symbolbuffer.left(byteswanted-1));
                symbolbuffer.remove(0,byteswanted-1);
                slip.sendpacket(ba);
            }

        }
        break;
    case JPKT_DATA_ACK:
  //      if(!pkt.length())ui->listWidget->addItem("ACK of data sent to Arduino");
        break;
    case JPKT_DATA_NACK:
        if(!pkt.length())ui->listWidget->addItem("NACK of data sent to Arduino");
        break;
    case JPKT_SYMBOL_PERIOD:
        if(pkt.length()==(4))
        {
            qint32 symbol_period=qFromLittleEndian<qint32>((uchar*)pkt.data());
            ui->statusBar->showMessage(((QString)"Symbol rate on Arduino is %1").arg(1000000/((double)symbol_period)));
            ui->listWidget->addItem(((QString)"Symbol rate on Arduino is %1").arg(1000000/((double)symbol_period)));
        }
        break;
    }

    while(ui->listWidget->count()>4)delete ui->listWidget->takeItem(0);
}


void MainWindow::Setport(QString port)
{
    slip.close();
    slip.setPortName(port);
    slip.setBaudRate(QSerialPort::Baud19200);
    if(!slip.open(QIODevice::ReadWrite))ui->statusBar->showMessage("Failed to open com port",10000);
    else
    {
        ui->statusBar->showMessage("Com port open",10000);
        QByteArray ba;
        ba.push_back((char)JPKT_ON);
        ba.push_back((char)0x00);
        slip.sendpacket(ba);
    }
}

void MainWindow::fillPortsInfo()
{
    ui->serialPortInfoListBox->clear();
    foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts())
    {
        ui->serialPortInfoListBox->addItem(info.portName());
    }
}

void MainWindow::on_serialPortInfoListBox_currentIndexChanged(const QString &port)
{
    Setport(port);
}

void MainWindow::on_txButton_clicked()
{
    //turn off dds
    QByteArray ba;
    ba.push_back((char)JPKT_ON);
    ba.push_back((char)0x00);
    slip.sendpacket(ba);

    //set frequency of dds
    quint32 value=ui->freq->value();
    ba.clear();
    ba.push_back((char)JPKT_FREQ);
    ba.push_back((char)((value>>(8*0))&0xFF));
    ba.push_back((char)((value>>(8*1))&0xFF));
    ba.push_back((char)((value>>(8*2))&0xFF));
    ba.push_back((char)((value>>(8*3))&0xFF));
    slip.sendpacket(ba);

    //set symbol rate of Arduino
    double fs=ui->symbolrate->value();
    value=1000000/fs;
    ba.clear();
    ba.push_back((char)JPKT_SYMBOL_PERIOD);
    ba.push_back((char)((value>>(8*0))&0xFF));
    ba.push_back((char)((value>>(8*1))&0xFF));
    ba.push_back((char)((value>>(8*2))&0xFF));
    ba.push_back((char)((value>>(8*3))&0xFF));
    slip.sendpacket(ba);

    //varicode text
    QVector<short> varicodedatabuffer;
    ba=ui->plainTextEdit->toPlainText().toLatin1();
    varicodedatabuffer.resize(VARICODE_MAX_BITS*ba.length());
    varicodedatabuffer.resize(varicode_encode(varicodedatabuffer.data(), ba.data(), VARICODE_MAX_BITS*ba.length(), ba.length(), 1));

    //test pi/2-BPSK
    uchar pi2offset=0;
    uchar pi2offset_change=0;//4; change to 4 and it will do pi/2-BPSK

    //modulate varicode as DBPSK where change means 0 and same means 1
    uchar lastphase=0;
    symbolbuffer.clear();
    int bit_of_char_ptr=0;
    uchar c=0;
    uchar phase;
    for(int bit_ptr=0;bit_ptr<varicodedatabuffer.length();bit_ptr++)
    {
        if(varicodedatabuffer.at(bit_ptr)==0)
        {
            phase=8-lastphase;//PSK31 0--> phase reversal
        }
        else
        {
            phase=lastphase;//PSK31 1--> same phase
        }
        lastphase=phase;
        pi2offset=pi2offset_change-pi2offset;

        c|=((phase+pi2offset)<<(4*bit_of_char_ptr));
        bit_of_char_ptr++;bit_of_char_ptr%=2;
        if(!bit_of_char_ptr)
        {
            symbolbuffer.push_back(c);
            c=0;
        }
    }
    while(bit_of_char_ptr)//fill remainder with zeros
    {
        phase=8-lastphase;//PSK31 0--> phase reversal
        lastphase=phase;
        pi2offset=pi2offset_change-pi2offset;
        c|=((phase+pi2offset)<<(4*bit_of_char_ptr));
        bit_of_char_ptr++;bit_of_char_ptr%=2;
        if(!bit_of_char_ptr)
        {
            symbolbuffer.push_back(c);
            c=0;
        }
    }

    for(int i=0;i<10;i++)
    qDebug()<<(uchar)symbolbuffer[i];

    //I know that the current symbol buffer on the arduino is 1024 and SLIP packets can be 255 bytes so lets que up 3 full packets of data
    // a better way would be ask the arduino how much space it has and the max SLIP packet size but i cant be bothered to do this.

    //que up some symbols
    ba.clear();
    ba.push_back((char)JPKT_Data);
    ba.push_back(symbolbuffer.left(254));
    symbolbuffer.remove(0,254);
    slip.sendpacket(ba);

    //que up some symbols
    ba.clear();
    ba.push_back((char)JPKT_Data);
    ba.push_back(symbolbuffer.left(254));
    symbolbuffer.remove(0,254);
    slip.sendpacket(ba);

    //que up some symbols
    ba.clear();
    ba.push_back((char)JPKT_Data);
    ba.push_back(symbolbuffer.left(254));
    symbolbuffer.remove(0,254);
    slip.sendpacket(ba);


    //start sending data
    ba.clear();
    ba.push_back((char)JPKT_ON);
    ba.push_back((char)0x01);
    slip.sendpacket(ba);
}

